==> rulee-ts-0.8.0.tgz: package/lib/esm/engine/builder/property-scalar-builder.js <==
import { SimpleValueProvider } from "../../provider/value-provider/simple-value-provider";
import { SelectValueConverter } from "../../value-converter/choices/select-value-converter";
import { ChoiceValueProvider } from "../../provider/value-provider/choices/choice-value-provider";
import { ObjectValueProvider } from "../../provider/value-provider/object-value-provider";
import { C } from "../../value-converter/common-value-converters";
import { DerivedValueProvider } from "../../provider/value-provider/derived-value-provider";
import { DerivedAsyncValueProvider } from "../../provider/value-provider/derived-async-value-provider";
import { EmptyValueFcns } from "../../provider/value-provider/empty-value-fcn";
import { ChoiceListConverter } from "../../value-converter/choices/choice-list-converter";
import { assertThat } from "../../util/assertions/assertions";
import { SelectValueProvider } from "../../provider/value-provider/choices/select-value-provider";
import { TypeaheadValueProvider } from "../../provider/value-provider/choices/typeahead-value-provider";
import { TypeaheadValueConverter } from "../../value-converter/choices/typeahead-value-converter";
import { upgradeAsPropertyWithChoices } from "../../properties/property-scalar-with-choices";
export class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new ChoiceValueProvider(choices, emptyChoice);
                const converter = new SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, undefined, config);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choices[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.sync(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            asyncDerived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            withPropertySource: (id, choicesSource, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new SelectValueProvider(choicesSource, emptyChoice);
                const converter = new SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], config, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choicesSource.getNonNullValue()[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, config) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), inputSourceProperty)({
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (config.minimumTextLength && text.length < config.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return config.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty, config);
            },
            withPropertySource: (id, inputSource, choicesSource, config) => {
                const provider = new TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], config, [inputSource, choicesSource]);
                return upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new DerivedValueProvider(dependencies, (deps) => config.derive(...deps), config === null || config === void 0 ? void 0 : config.inverse);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            },
            async: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new DerivedAsyncValueProvider(dependencies, (deps) => config.deriveAsync(...deps), config === null || config === void 0 ? void 0 : config.inverseAsync);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            }
        };
        assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        throw new Error('Not supported, yet');
    }
    simpleProperty(id, valueConverter, config) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    stringProperty(id, config) {
        var _a, _b;
        config = this.defaultInitValue('', config);
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.string.identity, undefined, config);
    }
    numberProperty(id, config) {
        var _a, _b;
        assertThat(() => !(config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) || !(config === null || config === void 0 ? void 0 : config.emptyValueFcn), () => `${id}: Provide either zeroIsConsideredAsEmpty or emptyValueFcn.`);
        config = this.defaultInitValue((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? 0 : null, config);
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : ((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? EmptyValueFcns.defaultEmptyValueFcn : EmptyValueFcns.numberEmptyValueFcn), (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.number.default, undefined, config);
    }
    booleanProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.booleanEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.boolean.default, undefined, config);
    }
    dateProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.date.iso, undefined, config);
    }
    defaultInitValue(initialValue, config) {
        if (!config) {
            return {
                initialValue
            };
        }
        if (config.initialValue === undefined) {
            config.initialValue = initialValue;
        }
        return config;
    }
    objectProperty(id, valueConverter, obj, config) {
        var _a;
        return this.propertyScalar(id, new ObjectValueProvider(obj, config.get, config.set), (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
}

==> rulee-ts-0.9.0.tgz: package/lib/esm/engine/builder/property-scalar-builder.js <==
import { SimpleValueProvider } from "../../provider/value-provider/simple-value-provider";
import { SelectValueConverter } from "../../value-converter/choices/select-value-converter";
import { ChoiceValueProvider } from "../../provider/value-provider/choices/choice-value-provider";
import { ObjectValueProvider } from "../../provider/value-provider/object-value-provider";
import { C } from "../../value-converter/common-value-converters";
import { DerivedValueProvider } from "../../provider/value-provider/derived-value-provider";
import { DerivedAsyncValueProvider } from "../../provider/value-provider/derived-async-value-provider";
import { EmptyValueFcns } from "../../provider/value-provider/empty-value-fcn";
import { ChoiceListConverter } from "../../value-converter/choices/choice-list-converter";
import { assertThat } from "../../util/assertions/assertions";
import { SelectValueProvider } from "../../provider/value-provider/choices/select-value-provider";
import { TypeaheadValueProvider } from "../../provider/value-provider/choices/typeahead-value-provider";
import { TypeaheadValueConverter } from "../../value-converter/choices/typeahead-value-converter";
import { upgradeAsPropertyWithChoices } from "../../properties/property-scalar-with-choices";
export class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new ChoiceValueProvider(choices, emptyChoice);
                const converter = new SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, undefined, config);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choices[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialState();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.sync(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            asyncDerived: (id, ...dependencies) => (config) => {
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(config);
                return this.select.withPropertySource(id, choicesSourceProperty, config);
            },
            withPropertySource: (id, choicesSource, config) => {
                var _a, _b;
                const emptyChoice = (_a = config === null || config === void 0 ? void 0 : config.emptyChoice) !== null && _a !== void 0 ? _a : this.defaultEmptyChoice;
                const provider = new SelectValueProvider(choicesSource, emptyChoice);
                const converter = new SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], config, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_b = choicesSource.getNonNullValue()[0]) === null || _b === void 0 ? void 0 : _b.value);
                prop.setToInitialState();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, config) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), inputSourceProperty)({
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (config.minimumTextLength && text.length < config.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return config.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty, config);
            },
            withPropertySource: (id, inputSource, choicesSource, config) => {
                const provider = new TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], config, [inputSource, choicesSource]);
                return upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new DerivedValueProvider(dependencies, (deps) => config.derive(...deps), config === null || config === void 0 ? void 0 : config.inverse);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            },
            async: (id, valueConverter, ...dependencies) => {
                return (config) => {
                    var _a;
                    const provider = new DerivedAsyncValueProvider(dependencies, (deps) => config.deriveAsync(...deps), config === null || config === void 0 ? void 0 : config.inverseAsync);
                    return this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                };
            }
        };
        assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        throw new Error('Not supported, yet');
    }
    simpleProperty(id, valueConverter, config) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    stringProperty(id, config) {
        var _a, _b;
        config = this.defaultInitValue('', config);
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.string.identity, undefined, config);
    }
    numberProperty(id, config) {
        var _a, _b;
        assertThat(() => !(config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) || !(config === null || config === void 0 ? void 0 : config.emptyValueFcn), () => `${id}: Provide either zeroIsConsideredAsEmpty or emptyValueFcn.`);
        config = this.defaultInitValue((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? 0 : null, config);
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : ((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? EmptyValueFcns.defaultEmptyValueFcn : EmptyValueFcns.numberEmptyValueFcn), (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.number.default, undefined, config);
    }
    booleanProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.booleanEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.boolean.default, undefined, config);
    }
    dateProperty(id, config) {
        var _a, _b;
        return this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.date.iso, undefined, config);
    }
    defaultInitValue(initialValue, config) {
        if (!config) {
            return {
                initialValue
            };
        }
        if (config.initialValue === undefined) {
            config.initialValue = initialValue;
        }
        return config;
    }
    objectProperty(id, valueConverter, obj, config) {
        var _a;
        return this.propertyScalar(id, new ObjectValueProvider(obj, config.get, config.set), (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, undefined, config);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
}
